unit ScreenSaver_ssaver;

interface

uses WinTypes, WinProcs, Graphics, Forms, Messages, Classes, Controls,
  SysUtils, _MnOglCanvas, _MNdevice, _MNComTypes,
  _MNOglTools, ImgList, _MNOglExTools, ComCtrls, ExtCtrls;

type
  TScrn = class(TForm)
    MNOglScene1: TMNOglScene;
    MNOglBox1: TMNOglBox;
    MnOglCanvas1: TMnOglCanvas;
    MNOglControl1: TMNOglControl;
    ImageList1: TImageList;
    MNOglBox2: TMNOglBox;
    MNOglBox3: TMNOglBox;
    MNOglBox4: TMNOglBox;
    MNOglBox5: TMNOglBox;
    MNOglBox6: TMNOglBox;
    MNOglLight1: TMNOglLight;
    MnOglSequencer1: TMnOglSequencer;
    MnOglTimer1: TMnOglTimer;
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormActivate(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure MnOglSequencer1Timer(Sender: TObject);
    procedure MnOglTimer1timer(Sender: TObject);
  private
    { Private declarations }
    Mouse : TPoint;
    InfoX, InfoY : integer;
    procedure StartSaver(var WinMsg : TMessage); message WM_USER+1;
    procedure StopSaver(var WinMsg : TMessage); message WM_USER+2;
    procedure GetPassword;
    procedure Trigger(Sender : TObject; var Done : Boolean);
  public
    { Public declarations }
    LoadingApp : Boolean;
  end;

type
  TSSMode = (ssSetPwd,ssPreview,ssConfig,ssRun);

var
  Scrn : TScrn;
  DesktopBitmap : TBitmap;

const
  SSMode      : TSSMode = ssRun;
  TestMode    : Boolean = False;

implementation

uses
  Registry;

const
  IgnoreCount : Integer = 0;

{$R *.DFM}

procedure CursorOff;
begin
  ShowCursor(False);
end;

procedure CursorOn;
begin
  ShowCursor(True);
end;

procedure TScrn.StartSaver(var WinMsg : TMessage);
begin
  // remains empty ...
end;

procedure TScrn.StopSaver(var WinMsg : TMessage);
begin
  CursorOn;
  GetPassword;
end;

procedure TScrn.GetPassword;
var
  MyMod     : THandle;
  PwdFunc   : function (Parent : THandle) : Boolean; stdcall;
  SysDir    : String;
  NewLen    : Integer;
  MyReg     : TRegistry;
  OkToClose : Boolean;
begin
  if (SSMode <> ssRun) or TestMode then begin
    Close;
    Exit;
  end;

  IgnoreCount := 5;
  OkToClose := False;
  MyReg := TRegistry.Create;
  MyReg.RootKey := HKEY_CURRENT_USER;
  if MyReg.OpenKey('Control Panel\Desktop',False) then begin
    try
      try
        ShowCursor(True);
        if MyReg.ReadInteger('ScreenSaveUsePassword') <> 0 then begin
          SetLength(SysDir,MAX_PATH);
          NewLen := GetSystemDirectory(PChar(SysDir),MAX_PATH);
          SetLength(SysDir,NewLen);
          if (Length(SysDir) > 0) and (SysDir[Length(SysDir)] <> '\') then
            SysDir := SysDir+'\';
          MyMod := LoadLibrary(PChar(SysDir+'PASSWORD.CPL'));
          if MyMod = 0 then
            OkToClose := True
          else begin
            PwdFunc := GetProcAddress(MyMod,'VerifyScreenSavePwd');
            if PwdFunc(Handle) then
              OkToClose := True;
            FreeLibrary(MyMod);
          end;
        end
        else
          OkToClose := True;
      finally
        ShowCursor(False);
      end;
    except
      OkToClose := True;
    end;
  end
  else
    OkToClose := True;

  MyReg.Free;

  if OkToClose then
    Close;
end;

procedure TScrn.Trigger(Sender : TObject; var Done : Boolean);
begin
  PostMessage(Handle,WM_USER+1,0,0);
end;

procedure TScrn.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  GetPassword;
end;

procedure TScrn.FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  if IgnoreCount > 0 then begin
    Dec(IgnoreCount);
    Exit;
  end;

  if (Mouse.X = -1) and (Mouse.Y = -1) then begin
    Mouse.X := X;
    Mouse.Y := Y;
  end
  else
    if (Abs(X-Mouse.X) > 2) and (Abs(Y-Mouse.Y) > 2) then begin
      Mouse.X := X;
      Mouse.Y := Y;
      GetPassword;
    end;
end;

procedure ILTexturesToGlobal (IL:TImageList);
VAR
  i:integer;
  B : TBitmap;
begin
  for i := 0 to IL.Count-1 do begin
    B := TBitmap.Create;
    IL.GetBitmap (i, B);
    MnAddBitmaptexture(IL.Name + '_' + IntToStr(i), B);
    B.Free;
  end;
end;

procedure TScrn.FormCreate(Sender: TObject);
begin
  LoadingApp := True;
end;

procedure TScrn.FormActivate(Sender: TObject);
var
  Dummy : Boolean;
begin
  if LoadingApp then begin
    LoadingApp := False;
    MNOglCanvas1.Changing := TRUE;
    Scrn.Color := clBlack;
    Scrn.Top := 0;
    Scrn.Left := 0;
    Scrn.Width := Screen.Width;
    Scrn.Height := Screen.Height;
    Mouse.X := -1;
    Mouse.Y := -1;
    InfoX := 10;
    InfoY := Screen.Height - Canvas.TextHeight ('W') - 5;

    ILTexturesToGlobal (ImageList1);
    MNOglBox1.Attributes.Texture.Picture := 'ImageList1_0.bmp';
    MNOglBox1.Attributes.Texture.Visible := TRUE;
    MNOglBox2.Attributes.Texture.Picture := 'ImageList1_1.bmp';
    MNOglBox2.Attributes.Texture.Visible := TRUE;
    MNOglBox3.Attributes.Texture.Picture := 'ImageList1_2.bmp';
    MNOglBox3.Attributes.Texture.Visible := TRUE;
    MNOglBox4.Attributes.Texture.Picture := 'ImageList1_3.bmp';
    MNOglBox4.Attributes.Texture.Visible := TRUE;
    MNOglBox5.Attributes.Texture.Picture := 'ImageList1_1.bmp';
    MNOglBox5.Attributes.Texture.Visible := TRUE;
    MNOglBox6.Attributes.Texture.Picture := 'ImageList1_4.bmp';
    MNOglBox6.Attributes.Texture.Visible := TRUE;

    Application.OnIdle := Trigger;
    SetWindowPos(Handle,HWND_TOPMOST,0,0,0,0,SWP_NOSIZE + SWP_NOMOVE);
    SystemParametersInfo(SPI_SCREENSAVERRUNNING,1,@Dummy,0);
    CursorOff;

    MNOglCanvas1.Changing := FALSE;

    Scrn.Visible := True;
    SetCapture(Scrn.Handle);
  end;
end;

procedure TScrn.FormMouseDown(Sender: TObject; Button: TMouseButton;
                              Shift: TShiftState; X, Y: Integer);
begin
  GetPassword;
end;

procedure TScrn.FormClose(Sender: TObject; var Action: TCloseAction);
var
  Dummy : Boolean;
begin
  CursorOn;
  SystemParametersInfo(SPI_SCREENSAVERRUNNING,0,@Dummy,0);
  Application.OnIdle := nil;
  ReleaseCapture;
end;

procedure TScrn.MnOglSequencer1Timer(Sender: TObject);
begin
  //
end;
procedure TScrn.MnOglTimer1timer(Sender: TObject);
begin
  MNOglScene1.RotateBy := MNOglScene1.RotateBy+2;
  InfoX := InfoX + 20;
  If InfoX > Screen.Width then
    InfoX := 10;
  MNOglCanvas1.MNCanvas.Brush.Style := bsClear;
  MNOglCanvas1.MNCanvas.Font.Color := clYellow;
  MNOglCanvas1.MNCanvas.TextOut (InfoX, InfoY, 'minais - MNOgl component collection http://www.minais.at');
  MNOglCanvas1.MNCanvas.Font.Color := clWhite;
end;

end.
_mncomtypes
